/// \file mil1553ud_library.c
/// \brief библиотка взаимодействия
/// реалитзация функций
#include "mil1553ud_library.h"

/// \brief паттерн имени девайса
#define MIL1553_STR_CH0_PATTERN                 "-ch-0"
/// \brief паттерн имени канала
#define MIL1553_STR_DEV_PATTERN                 "mil1553dev"
/// \brief каталог с каналами
#define MIL1553_STR_DEV                         "/dev"

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА MIL1553. ФУНКЦИИ.
/// /// /// /// /// /// /// /// /// /// /// ///

unsigned int mil1553_readDma(int fd, struct MIL1553_DMA_CONTAINER_BLOCKS* container) {
    DMA_READ_BLOCK block;
    unsigned int ret = GOOD_CODE;
    unsigned int countReadyReadBlocks = 0;
    countReadyReadBlocks = mil1553ud_getCountDmaBlocksReadyRead(fd, &ret);
    container->mCount = 0;
    if (ret) {
        goto END;
    }
    if (countReadyReadBlocks == 0) {
        goto END;
    }
    if (countReadyReadBlocks >= MIL1553_DMA_CONTAINER_BLOCKS_MAX_SIZE)
        container->mCount = MIL1553_DMA_CONTAINER_BLOCKS_MAX_SIZE;
    else
        container->mCount = countReadyReadBlocks;
    block.countBlocks = container->mCount;
    block.length = block.countBlocks * MIL1553_DMA_BLOCK_SIZE;
    block.data = (unsigned char*)malloc(block.length);
    ret = ioctl(fd, IOCTL_READ_BLOCKS_RAW_DMA, &block);
    if (ret) {
        goto FREE;
    }
    memcpy(container->mBlocks, block.data, block.length);
FREE:
    free(block.data);
END:
    return ret;
}

unsigned int mil1553_manChannel(int fd, unsigned int op) {
    unsigned int ret = GOOD_CODE;
    unsigned int numChannel = mil1553ud_getNumberChannel(fd);
    SADDR_DATA_BIT_MASK reg;
    reg.daddr = MIL_REG(numChannel, CTRL_REG_PCI_CH);
    reg.mask = (1<<23);

    if (MIL_DEV_CHANNEL_ON == op) {
        reg.data = (1<<23);
    } else {
        reg.data = (0<<23);
    }
    ret = mil1553_modifyReg(fd, &reg);
    //ret = ioctl(fd, IOCTL_DEV_CHANNEL_WORK, &op);
    return ret;
}

unsigned int mil1553_setRole(int fd, unsigned int role) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_SWITCH_MODE, &role);
    return ret;
}

int mil1553_getCountChannels() {
    int count = 0;
    DIR* dir = opendir(MIL1553_STR_DEV);
    if (dir) {
        struct dirent *direntries;
        while (((direntries = readdir(dir)) != NULL)) {
            char* isFinded = strstr(direntries->d_name, MIL1553_STR_DEV_PATTERN);
            if (isFinded != NULL) {
//                printf("%s\n", direntries->d_name);
                count++;
            }
        }
        closedir(dir);
    }
    return count;
}

int mil1553_getCountDevices(void) {
    int count = 0;
    DIR* dir = opendir(MIL1553_STR_DEV);
    if (dir) {
        struct dirent *direntries;
        while (((direntries = readdir(dir)) != NULL)) {
            char* isFindedChannel = strstr(direntries->d_name, MIL1553_STR_DEV_PATTERN);
            char* isFindedChannel0 = strstr(direntries->d_name, MIL1553_STR_CH0_PATTERN);
            if ((isFindedChannel != NULL) && (isFindedChannel0 != NULL)) {
//                printf("device: %s\n", direntries->d_name);
                count++;
            }
        }
        closedir(dir);
    }
    return count;
}

void mil1553_getArrayDevices(struct TMil1553Device** devices, int* length) {
    *length = mil1553_getCountDevices();
    *devices = malloc(sizeof(struct TMil1553Device) * *length);
    int index = 0;
    DIR* dir = opendir(MIL1553_STR_DEV);
    if (dir) {
        struct dirent *direntries;
        while (((direntries = readdir(dir)) != NULL)) {
            char* isFindedChannel = strstr(direntries->d_name, MIL1553_STR_DEV_PATTERN);
            char* isFindedChannel0 = strstr(direntries->d_name, MIL1553_STR_CH0_PATTERN);
            if ((isFindedChannel != NULL) && (isFindedChannel0 != NULL)) {
                snprintf((*devices)[index].mDev.mNameChannel, CHANNEL_NAME_LENGTH, "%s/%s", MIL1553_STR_DEV, isFindedChannel);
                (*devices)[index].mDev.mFD = mil1553_open((*devices)[index].mDev.mNameChannel);
                mil1553_getDeviceInfo((*devices)[index].mDev.mFD, &(*devices)[index].mInfo);
                mil1553_close((*devices)[index].mDev.mFD);
                index++;
            }
        }
        closedir(dir);
    }
}

/// \brief паттерн считывания номера устройства - старая версия
#define MIL1553_STR_CHAN_LEGACY_PATTERN         MIL1553_STR_DEV "/" MIL1553_STR_DEV_PATTERN "-%d"
/// \brief паттерн считывания номера устройства - новая версия, с пропуском типа подключения (pci/eth/usb)
#define MIL1553_STR_CHAN_CURRENT_PATTERN        MIL1553_STR_DEV "/" MIL1553_STR_DEV_PATTERN "-%*c%*c%*c-%d"

int mil1553_getNumberDevice(struct TMil1553Channel * channel) {
    int number = -1;

    sscanf(channel->mNameChannel, MIL1553_STR_CHAN_LEGACY_PATTERN, &number);
    if(number == -1)
        sscanf(channel->mNameChannel, MIL1553_STR_CHAN_CURRENT_PATTERN, &number);

    return number;
}

void mil1553_getArrayChannels(struct TMil1553Channel **channels, int *length) {
    *length = mil1553_getCountChannels();
    *channels = malloc(sizeof(struct TMil1553Channel) * *length);
    int index = 0;
    DIR* dir = opendir(MIL1553_STR_DEV);
    if (dir) {
        struct dirent *direntries;
        while (((direntries = readdir(dir)) != NULL)) {
            char* isFinded = strstr(direntries->d_name, MIL1553_STR_DEV_PATTERN);
            if (isFinded != NULL) {
                snprintf((*channels)[index].mNameChannel, CHANNEL_NAME_LENGTH, "%s/%s", MIL1553_STR_DEV, isFinded);
                index++;
            }
        }
        closedir(dir);
    }
}

void mil1553_getArrayChannelsByDevice(struct TMil1553Device* dev, struct TMil1553Channel **channels, int *length) {
    int index = 0;
    struct TMil1553Channel* all_channels;
    int current_dev_number = 0;
    int all_ch_len = 0;
    current_dev_number = mil1553_getNumberDevice(&(dev->mDev));
    mil1553_getArrayChannels(&all_channels, &all_ch_len);
    *length = 0;
    for (index = 0; index < all_ch_len; index++) {
        int dev_number = mil1553_getNumberDevice(&(all_channels[index]));
        if (dev_number == current_dev_number)
            (*length)++;
    }
    *channels = malloc(sizeof(struct TMil1553Channel) * *length);
    int cur_index = 0;
    for (index = 0; index < all_ch_len; index++) {
        int dev_number = mil1553_getNumberDevice(&(all_channels[index]));
        if (dev_number == current_dev_number) {
            memcpy(&(*channels)[cur_index++], &(all_channels[index]), sizeof(struct TMil1553Channel));
        }
    }
    free(all_channels);
}

unsigned int mil1553_writeReg(int fd, SADDR_DATA* reg) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_WRITE_REG, reg);
    return ret;
}

unsigned int mil1553_readReg(int fd, SADDR_DATA* reg) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_READ_REG, reg);
    return ret;
}

unsigned int mil1553_modifyReg(int fd, SADDR_DATA_BIT_MASK* reg) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_WRITE_REG_BIT_MASK, reg);
    return ret;
}

unsigned int mil1553_getDeviceInfo(int fd, VERSION* info){
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_VERSION, info);
    return ret;
}

unsigned int mil1553_getDriverVersion(int fd, unsigned int* version) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_VERSION_DRIVER, version);
    return ret;
}

unsigned int mil1553_getDevicePciInfo(int fd, PCI_LOCATION* pciInfo) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_PCI_LOCATION, pciInfo);
    return ret;
}

unsigned int mil1553_clearDma(int fd) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_CLEAR_DMA);
    return ret;
}

unsigned int mil1553_manDma(int fd, int op) {
    unsigned int ret = GOOD_CODE;
    if (op) {
        ret = ioctl(fd, IOCTL_ENABLE_DMA);
    } else {
        ret = ioctl(fd, IOCTL_DISABLE_DMA);
    }
    return ret;
}

unsigned int mil1553_switchBus(int fd, unsigned int bus) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_SWITCH_BUS, &bus);
    return ret;
}

#define MIL1553_BC_RAM_WRITE_SIZE_BLOCK      128

unsigned int mil1553_bc_writeDataToRam(int fd, enum EMil1553BcTypeDataRam type, unsigned int* data, int length) {
    BC_RAM_BLOCK* blockWrite;
    BC_RAM_BLOCK* blockRead;
    int ret = GOOD_CODE;
    unsigned int index = 0;
    int i = 0;
    unsigned int countBlocks = 0;
    unsigned int sizeTailBlock = 0;
    if ( (length > MIL1553_BC_MAX_SIZE_RAM_NONDATA) && (type != Mil1553BcTypeData) ) {
        ret = MIL1553_BC_WRT_DATA_RAM_ERROR_MAX_LEN;
    } else if ( (length > MIL1553_BC_MAX_SIZE_RAM_DATA) && (type == Mil1553BcTypeData) ) {
        ret = MIL1553_BC_WRT_DATA_RAM_ERROR_MAX_LEN;
    }
    if (ret != GOOD_CODE)
        return ret;
    blockWrite = mil1553ud_newBcRamBlock(type, MIL1553_BC_RAM_WRITE_SIZE_BLOCK, 0);
    blockRead = mil1553ud_newBcRamBlock(type, MIL1553_BC_RAM_WRITE_SIZE_BLOCK, 0);
    countBlocks = length / MIL1553_BC_RAM_WRITE_SIZE_BLOCK;
    sizeTailBlock = length % MIL1553_BC_RAM_WRITE_SIZE_BLOCK;

    for (index = 0; index < countBlocks; index++) {
        blockWrite->shift = index * MIL1553_BC_RAM_WRITE_SIZE_BLOCK;
        blockRead->shift = blockWrite->shift;
        memcpy(blockWrite->dwords, data + blockWrite->shift, MIL1553_BC_RAM_WRITE_SIZE_BLOCK * sizeof(unsigned int));
        // записываем микропрограмму
        ret = ioctl(fd, IOCTL_WRITE_BC_RAM, blockWrite);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_WRT;
            goto END;
        }
        ret = ioctl(fd, IOCTL_READ_BC_RAM, blockRead);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_RD;
            goto END;
        }
        // пословное сравнение
        for (i = 0; i < MIL1553_BC_RAM_WRITE_SIZE_BLOCK; i++) {
            unsigned int valWrt = blockWrite->dwords[i], valRd = blockRead->dwords[i];
            if (valWrt != valRd) {
                ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_CHECK;
                goto END;
            }
        }
    }
    if (sizeTailBlock) {
        blockWrite->shift = index * MIL1553_BC_RAM_WRITE_SIZE_BLOCK;
        for (i = 0; i < MIL1553_BC_RAM_WRITE_SIZE_BLOCK; i++) {
            if (i >= sizeTailBlock)
                blockWrite->dwords[i] = 0;
            else
                blockWrite->dwords[i] = data[blockWrite->shift + i];
        }
        // записываем микропрограмму
        ret = ioctl(fd, IOCTL_WRITE_BC_RAM, blockWrite);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_WRT;
            goto END;
        }
        ret = ioctl(fd, IOCTL_READ_BC_RAM, blockRead);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_RD;
            goto END;
        }
        // пословное сравнение
        for (i = 0; i < MIL1553_BC_RAM_WRITE_SIZE_BLOCK; i++) {
            unsigned int valWrt = blockWrite->dwords[i], valRd = blockRead->dwords[i];
            if (valWrt != valRd) {
                ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_CHECK;
                goto END;
            }
        }
    }
END:
    // освобождаем буфер микропрограммы
    mil1553ud_freeBcRamBlock(blockWrite);
    mil1553ud_freeBcRamBlock(blockRead);
    return ret;
}

unsigned int mil1553_rt_manSubaddresses(int fd, unsigned int direction, unsigned int subadr_mask, unsigned int action_mask) {
    unsigned int ret = GOOD_CODE;
    RT_BUF_MAN_EN rtBufMan;
    rtBufMan.direction = direction;
    rtBufMan.subaddress_mask = subadr_mask; // меняем с 1 по 30 подадрес
    rtBufMan.action_mask = action_mask; // включаем на приём с 1 по 30 подадрес
    ret = ioctl(fd, IOCTL_RT_BUF_MAN_EN, &rtBufMan);
    return ret;
}

unsigned int mil1553_rt_setAddress(int fd, unsigned char address) {
    unsigned int ret = GOOD_CODE;
    ret = ioctl(fd, IOCTL_SET_ADDRESS, &address);
    return ret;
}

unsigned int mil1553_rt_manSubadressTransmitBufferMode(int fd, unsigned char subaddress, unsigned int mode) {
    unsigned int ret = GOOD_CODE;
    RT_TR_MODE rt_tr_mode;
    rt_tr_mode.mode = mode;
    rt_tr_mode.subaddress = subaddress;
    ret = ioctl(fd, IOCTL_SET_RT_TR_MODE, &rt_tr_mode);
    return ret;
}

unsigned int mil1553_rt_manSubaddressTransmitBufferReady(int fd, unsigned char subaddress, unsigned int cmd_buf) {
    unsigned int ret = GOOD_CODE;
    RT_TR_MODE_BUF rdBuf;
    rdBuf.cmd_buf = cmd_buf;
    rdBuf.subaddress = subaddress;
    ret = ioctl(fd, IOCTL_SET_RT_TR_BUF_READY, &rdBuf);
    return ret;
}

unsigned int mil1553_rt_writeDataToSubaddressTransmit(int fd, unsigned int num_buf, unsigned char subaddress, struct TRtSubaddressData data) {
    unsigned int ret = GOOD_CODE;
    RT_TR_BUF_SUBADDR bufSub;
    bufSub.num_buf = num_buf;
    bufSub.subaddress = subaddress;
    memcpy(bufSub.data_words, data.mWords, data.mCount * 2);
    ret = ioctl(fd, IOCTL_WR_BLOCK_BUF_SUBADR, &bufSub);
    return ret;
}



/*-------------------------------------------------------------------------------------------------------*/

unsigned int mil1553ud_getNumberChannel(int fd)
{
    unsigned int numCh = 0;         // номер канала
    if (ioctl(fd, IOCTL_GET_NUMBER_CH, &numCh))
        return LIB_BAD_VALUE;   // ошибка
    else
        return numCh;
}

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ДМА

unsigned int mil1553ud_getCountDmaBlocksReadyRead(int fd, unsigned int* codeError )
{
    unsigned int countBlocks = 0;   // кол-во блоков
    *codeError = ioctl(fd, IOCTL_GET_NBLOCK_RAW_DMA, &countBlocks);
    int code = *codeError ;
    if (code)
        return LIB_BAD_VALUE;   // ошибка
    else
        return countBlocks;
}

DMA_READ_BLOCK* mil1553ud_newDmaReadBlock(unsigned int countDmaBlocks)
{
    DMA_READ_BLOCK* block = (DMA_READ_BLOCK*)malloc(sizeof(DMA_READ_BLOCK));
    block->countBlocks = countDmaBlocks;
    block->length = block->countBlocks * MIL_DMA_BLOCK_SIZE;
    block->data = (unsigned char*)malloc(block->length);
    return block;
}

void mil1553ud_freeDmaReadBlock(DMA_READ_BLOCK *block)
{
    if ( block != NULL ) {
        if (block->data != NULL) {
            free(block->data);
        }
        free(block);
    }
}

DMA_READ_BLOCK* mil1553ud_readAllDmaBlocksReadyRead(int fd, unsigned int* cntBlocks)
{
    DMA_READ_BLOCK* block = NULL;   // контейнер дма блоков
    unsigned int countBlocks = 0;   // кол-во блоков
    unsigned int codeError = 0;
    countBlocks = mil1553ud_getCountDmaBlocksReadyRead(fd, &codeError);
    *cntBlocks = countBlocks;
    if ( countBlocks == 0 ) {
        return block;
    } else {
        if (codeError != 0) {
            return NULL;
        }
        block = mil1553ud_newDmaReadBlock(countBlocks);
        if (ioctl(fd, IOCTL_READ_BLOCKS_RAW_DMA, block)) { // ошибка
            mil1553ud_freeDmaReadBlock(block);
            return NULL;
        } else {
            return block;
        }
    }
}

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ПРЕРЫВАНИЙ

unsigned int mil1553ud_getCountInterruptBlocksReadyRead(int fd)
{
    unsigned int countBlocks = 0;   // кол-во блоков
    if (ioctl(fd, IOCTL_GET_COUNT_INTERRUPT_BLOCKS, &countBlocks))
        return LIB_BAD_VALUE; // ошибка
    else
        return countBlocks;
}

unsigned int mil1553ud_readInterruptBlocks(int fd, INTERRUPT_BLOCK_BUFFER* block)
{
    unsigned int resutl = GOOD;
    resutl = ioctl(fd, IOCTL_READ_INTERRUPT_BLOCKS, block);
    return resutl;
}

/// \remark РЕЖИМ КШ

unsigned int mil1553ud_bcProgram(int fd, unsigned int action)
{
    SADDR_DATA_BIT_MASK bitMask;    // битовая маска регистра
    unsigned int numCh = mil1553ud_getNumberChannel(fd);
    bitMask.daddr = MIL_REG(numCh, BC_CONF_REG_PCI_CH);     // адрес регистра
    bitMask.mask = 1 << 15; // изменяем только 15 бит - BCSTRT
    if (action == (unsigned int)BC_PROGRAM_START) {
        bitMask.data = 1 << 15; // BCSTRT в 1
    } else {
        bitMask.data = 0;
    }
    if (ioctl(fd, IOCTL_WRITE_REG_BIT_MASK, &bitMask))
        return LIB_BAD_VALUE; // ошибка
    else
        return LIB_GOOD;
}

unsigned int mil1553ud_bcSetStartInstructionAddress(int fd, unsigned int startNumber)
{
    SADDR_DATA_BIT_MASK bitMask;    // битовая маска регистра
    unsigned int numChannel = mil1553ud_getNumberChannel(fd);
    bitMask.daddr = MIL_REG(numChannel, START_ADR_INSTR_PTR_CH);    // адрес регистра стартовой инструкции
    bitMask.mask = 0xFFF << 16;
    bitMask.data = (startNumber << 16) & (0xFFF << 16);
    if (ioctl(fd, IOCTL_WRITE_REG_BIT_MASK, &bitMask))
        return LIB_BAD_VALUE; // ошибка
    else
        return LIB_GOOD;
}

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ

BC_RAM_BLOCK *mil1553ud_newBcRamBlock(unsigned int type, unsigned int sizeInDwords, unsigned int shiftInDwords)
{
    BC_RAM_BLOCK* block = (BC_RAM_BLOCK*)malloc(sizeof(BC_RAM_BLOCK));
    block->type = type;
    block->length = sizeInDwords;
    block->shift = shiftInDwords;
    block->dwords = (unsigned int*)malloc(block->length * DWORD_SIZE);
    return block;
}

void mil1553ud_freeBcRamBlock(BC_RAM_BLOCK *block)
{
    if (block != NULL) {
        if (block->dwords != NULL) {
            free(block->dwords);
        }
        free(block);
    }
}

/// \remark РАСШИФРОВКА БЛОКОВ ДМА

unsigned int mil1553ud_dmaBlock_getTypeTransaction(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 1) & 0xF;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 27) & 0xF;
    }
}

unsigned int mil1553ud_dmaBlock_isSuccessTransaction(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 28) & 0x1) ? MIL1553UD_TRUE : MIL1553UD_FALSE;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 26) & 0x1) ? MIL1553UD_FALSE : MIL1553UD_TRUE;
    }
}

unsigned int mil1553ud_dmaBlock_getFreeTimer(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 1 * DWORD_SIZE);
        return *dword;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 4 * DWORD_SIZE);
        return *dword;
    }
}

unsigned int mil1553ud_dmaBlock_getCountServiceWords16(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 11) & 0xF;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 11) & 0xF;
    }
}

unsigned int mil1553ud_dmaBlock_getCountDataWords16(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 5) & 0x3F;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 5) & 0x3F;
    }
}

unsigned int mil1553ud_dmaBlock_getActiveBus(unsigned int typeDmaBlock, unsigned char *dmaBlock)
{
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {   // КШ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ( (*dword >> 15) & 0x1 ) ? MIL1553_BUS_B : MIL1553_BUS_A ;
    } else {    // ОУ
        unsigned int* dword = (unsigned int*)(dmaBlock + 0 * DWORD_SIZE);
        return ( (*dword >> 31) & 0x1 ) ? MIL1553_BUS_B : MIL1553_BUS_A ;
    }
}

unsigned char *mil1553ud_dmaBlock_getPtrDataPart(unsigned char *dmaBlock)
{
    return dmaBlock + MIL_DMA_BLOCK_SIZE/2;
}

unsigned int mil1553ud_dmaBlock_tdModeSubaddress(unsigned char *dmaBlock)
{
    return *((unsigned int*)dmaBlock) & 0x1F;
}

